/* Ani.js
 * 
 * Version 0.1 (alpha)
 * 
 * Copyright 2012 Lars Ebert
 * http://www.advitum.de
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * This copyright-header must not be removed!
 *
 */

function Animation(options) {
	var thisAnimation = this;
	
	options = join(options, {
		modifier: function(scene, node, amount, frames) {
			
		},
		movements: [],
		start: false,
		loop: false
	});
	
	thisAnimation.loop = options.loop;
	thisAnimation.running = options.start;
	thisAnimation.movements = options.movements;
	thisAnimation.step = 0;
	thisAnimation.frame = 0;
	thisAnimation.modifier = options.modifier;
	
	thisAnimation.update = function() {
		if(thisAnimation.running) {
			if(typeof thisAnimation.movements[thisAnimation.step] == "function") {
				thisAnimation.movements[thisAnimation.step](thisAnimation.scene, thisAnimation.node);
				
				thisAnimation.step++;
				
				if(thisAnimation.step >= thisAnimation.movements.length) {
					thisAnimation.step = 0;
					if(!thisAnimation.loop) {
						thisAnimation.running = false;
					}
				}
			} else {
				if(thisAnimation.movements[thisAnimation.step].duration > 0) {
					var frames = Math.ceil(thisAnimation.movements[thisAnimation.step].duration/thisAnimation.scene.fps);
					
					thisAnimation.frame++;
					thisAnimation.modifier(thisAnimation.scene, thisAnimation.node, thisAnimation.movements[thisAnimation.step].movements, frames);
					
					if(thisAnimation.frame >= frames)
					{
						thisAnimation.frame = 0;
						thisAnimation.step++;
						
						if(thisAnimation.step >= thisAnimation.movements.length) {
							thisAnimation.step = 0;
							if(!thisAnimation.loop) {
								thisAnimation.running = false;
							}
						}
					}
				} else {
					thisAnimation.modifier(thisAnimation.scene, thisAnimation.node, thisAnimation.movements[thisAnimation.step].movements, 1);
					
					thisAnimation.step++;
					
					if(thisAnimation.step >= thisAnimation.movements.length) {
						thisAnimation.step = 0;
						if(!thisAnimation.loop) {
							thisAnimation.running = false;
						}
					}
				}
			}
		}
	}
	
	thisAnimation.start = function() {
		thisAnimation.running = true;
	}
	
	thisAnimation.stop = function() {
		thisAnimation.running = false;
	}
}

function Node(options) {
	var thisNode = this;
	
	options = join(options, {
		layer: 1,
		x: 0,
		y: 0,
		rotation: {},
		func: 0,
		image: 0,
		animations: {},
		nodes: {},
		additional: {}
	});
	
	options.rotation = join(options.rotation, {
		x: 0,
		y: 0,
		angle: 0
	});
	
	$.each(options.additional, function(key, value) {
		thisNode[key] = value;
	});
	
	thisNode.nodes = options.nodes;
	thisNode.animations = options.animations;
	$.each(thisNode.animations, function(key, animation) {
		animation.node = thisNode;
	});
	thisNode.layer = options.layer;
	
	thisNode.x = options.x;
	thisNode.y = options.y;
	
	if(options.func != 0) {
		thisNode.func = options.func;
	}
	if(options.image != 0) {
		thisNode.image = options.image;
	}
	
	thisNode.rotation = options.rotation;
	
	thisNode.draw = function(x, y) {
		if(typeof thisNode.scene.contexts[thisNode.layer] == "undefined") {
			thisNode.scene.newCanvas(thisNode.layer);
		}
		
		$.each(thisNode.animations, function(name, animation) {
			animation.update();
		});
		
		$.each(thisNode.scene.contexts, function(layer, context) {
			context.translate(x + thisNode.x + thisNode.rotation.x, y + thisNode.y + thisNode.rotation.y);
			//drawEllipse(0, 0, 5, 5, '#ff0000', context);
			context.rotate(2*Math.PI*thisNode.rotation.angle);
			context.translate(-(x + thisNode.x + thisNode.rotation.x), -(y + thisNode.y + thisNode.rotation.y));
		});
		
		if(typeof thisNode.image != "undefined") {
			thisNode.scene.contexts[thisNode.layer].drawImage(thisNode.scene.images[thisNode.image], x + thisNode.x, y + thisNode.y);
		}
		if(typeof thisNode.func == "function") {
			thisNode.func(x, y, thisNode);
		}
		
		$.each(thisNode.nodes, function(name, node) {
			node.draw(x + thisNode.x, y + thisNode.y);
		});
		
		$.each(thisNode.scene.contexts, function(layer, context) {
			context.translate(x + thisNode.x + thisNode.rotation.x, y + thisNode.y + thisNode.rotation.y);
			context.rotate(-2*Math.PI*thisNode.rotation.angle);
			context.translate(-(x + thisNode.x + thisNode.rotation.x), -(y + thisNode.y + thisNode.rotation.y));
		});
	}
	
	thisNode.setScene = function(scene) {
		thisNode.scene = scene;
		$.each(thisNode.nodes, function(key, node) {
			node.setScene(scene);
		});
		$.each(thisNode.animations, function(key, animation) {
			animation.scene = scene;
		});
	}
	
	thisNode.addNode = function(name, newNode) {
		newNode.setScene(thisNode.scene);
		thisNode.nodes[name] = newNode;
	}
	
	thisNode.addAnimation = function(name, newAnimation) {
		newAnimation.node = thisNode;
		newAnimation.scene = thisNode.scene;
		thisNode.animations[name] = newAnimation;
	}
	
	thisNode.removeNode = function(name) {
		delete thisNode.nodes[name];
	}
	
	thisNode.removeAnimation = function(name) {
		delete thisNode.animations[name];
	}
}

function Scene(width, height) {
	var thisScene = this;
	
	thisScene.images = {};
	thisScene.sounds = {};
	thisScene.imagesLoaded = 0;
	thisScene.nodes = {};
	thisScene.animations = {};
	
	thisScene.width = width;
	thisScene.height = height;
	
	thisScene.fps = 30;
	
	thisScene.setElement = function(element) {
		thisScene.element = element;
		thisScene.canvases = {};
		thisScene.contexts = {};
		
		thisScene.element.css({
			position: 'relative',
			width: thisScene.width + 'px',
			height: thisScene.height + 'px'
		});
	}
	
	thisScene.newCanvas = function(layer) {
		thisScene.element.append('<canvas id="layer' + layer + '" style="position: absolute; top: 0px; left: 0px; z-index: ' + layer + ';" width="' + thisScene.width + '" height="' + thisScene.height + '"></canvas>');
		thisScene.canvases[layer] = $('#layer' + layer)[0];
		thisScene.contexts[layer] = thisScene.canvases[layer].getContext("2d");
	}
	
	thisScene.loadImages = function(images) {
		$.each(images, function(id, name) {
			thisScene.images[name] = new Image();
			thisScene.images[name].onload = function() {
				thisScene.imagesLoaded += 1;
				
				if(length(thisScene.images) == thisScene.imagesLoaded) {
					thisScene.init();
					setInterval(thisScene.draw, 1000/thisScene.fps);
				}
			}
			thisScene.images[name].src = "images/" + name + ".png";
		});
	}
	
	thisScene.loadSounds = function(sounds) {
		$.each(sounds, function(id, name) {
			thisScene.sounds[name] = new Audio();
			thisScene.element.prepend(thisScene.sounds[name]);
			if(thisScene.sounds[name].canPlayType("audio/ogg").match(/maybe|probably/i)) {
				thisScene.sounds[name].src = 'sounds/' + name + '.ogg';
			} else {
				thisScene.sounds[name].src = 'sounds/' + name + '.mp3';
			}
		});
	}
	
	thisScene.playSound = function(name) {
		thisScene.sounds[name].play();
	}
	
	thisScene.init = function() {
		
	}
	
	thisScene.update = function() {
		
	}
	
	thisScene.addNode = function(name, newNode) {
		newNode.setScene(thisScene);
		thisScene.nodes[name] = newNode;
	}
	
	thisScene.addAnimation = function(name, newAnimation) {
		newAnimation.node = thisScene;
		newAnimation.scene = thisScene;
		thisScene.animations[name] = newAnimation;
	}
	
	thisScene.removeNode = function(name) {
		delete thisScene.nodes[name];
	}
	
	thisScene.removeAnimation = function(name) {
		delete thisScene.animations[name];
	}
	
	thisScene.draw = function() {
		$.each(thisScene.contexts, function(layer, context) {
			context.clearRect(0, 0, thisScene.width, thisScene.height);
		});
		
		$.each(thisScene.animations, function(name, animation) {
			animation.update();
		});
		
		thisScene.update();
		
		$.each(thisScene.nodes, function(name, node) {
			node.draw(0, 0);
		});
	}
}

function length(obj) {
    var size = 0, key;
    for (key in obj) {
        if (obj.hasOwnProperty(key)) size++;
    }
    return size;
}

function join(options, defaults) {
	$.each(defaults, function(key, value) {
		if(typeof options[key] == "undefined") {
			options[key] = value;
		}
	});
	
	return options;
}

function drawEllipse(centerX, centerY, width, height, color, context) {
	context.beginPath();
	
	context.moveTo(centerX, centerY - height/2);
	
	context.bezierCurveTo(
	centerX + width/2, centerY - height/2,
	centerX + width/2, centerY + height/2,
	centerX, centerY + height/2);
	
	context.bezierCurveTo(
	centerX - width/2, centerY + height/2,
	centerX - width/2, centerY - height/2,
	centerX, centerY - height/2);
	
	context.fillStyle = color;
	context.fill();
	context.closePath();	
}